"""Initialize testing module, specify used module versions."""
import gi

gi.require_version("Adw", "1")
gi.require_version("Gdk", "4.0")
gi.require_version("Graphs", "1")
gi.require_version("Gtk", "4.0")
