@define-color card_border @borders;

inline-stack-switcher {
  box-shadow: inset 0 0 0 1px @borders;
}

inline-stack-switcher button:checked {
  box-shadow: inset 0 0 0 1px @borders,
              0 1px 3px 1px alpha(black, .07),
              0 2px 6px 2px alpha(black, .03);
}
