// SPDX-License-Identifier: GPL-3.0-or-later
namespace Graphs {

    public const string[] LIMIT_NAMES = {
        "min-bottom", "max-bottom", "min-top", "max-top",
        "min-left", "max-left", "min-right", "max-right",
    };

    public const string[] DIRECTION_NAMES = {
        "bottom", "top", "left", "right"
    };

    public const string[] ACTION_NAMES = {
        "quit",
        "help",
        "about",
        "figure_settings",
        "add_data",
        "add_equation",
        "select_all",
        "select_none",
        "undo",
        "redo",
        "optimize_limits",
        "view_back",
        "view_forward",
        "export_data",
        "export_figure",
        "new_project",
        "save_project",
        "save_project_as",
        "smoothen_settings",
        "open_project",
        "delete_selected",
        "zoom_in",
        "zoom_out"
    };
}
